@@ -23,6 +23,33 @@ methods, which allow Template constructor options to be embedded in the template
 string itself.
 
 Other functions are exposed for tools which may want to process templates.
+
+Unicode
+-------
+
+JSON Template can work on unicode strings or byte strings.  The template parser
+and expansion loop don't care which type they get.
+
+However, it's up to the caller to ensure that they get *compatible* types.
+Python auto-conversion can make this a bit confusing.
+
+If you have a byte string template and a dictionary with byte strings, expansion
+will work:
+
+'Hello {name}' +  {name: '\xb5'}  ->  'Hello \xb5'
+
+If you have a unicode template and unicode data in the dictionary, it will work:
+
+u'Hello {name}' +  {name: u'\u00B5'}  ->  u'Hello \u00B5'
+
+If you have a unicode template and byte string data, Python will try to decode
+the byte strings using the utf-8 encoding.  This may not be possible, in which
+case you'll get a UnicodeDecodeError.
+
+u'Hello {name}' +  {name: 'there'}  ->  'Hello there'
+u'Hello {name}' +  {name: '\xb5'}     ->  ERROR: \xb5 is not decodable as ASCII
+
+Mixing types may incur a performance penalty.
 """
 
 __author__ = 'Andy Chu'
@@ -666,7 +693,9 @@ class _ScopedContext(object):
 
 
 def _ToString(x):
-  # Some cross-language values for primitives
+  """The default default formatter!."""
+  # Some cross-language values for primitives.  This is tested in
+  # jsontemplate_test.py.
   if x is None:
     return 'null'
   if isinstance(x, basestring):
@@ -753,6 +782,9 @@ _DEFAULT_FORMATTERS = {
     # this could be lambda x: json.dumps(x, indent=2), but here we want to be
     # compatible to Python 2.4.
     'str': _ToString,
+    # Python-specific representation for safely debugging any value as an ASCII
+    # string (unicode can cause issues when using 'str')
+    'repr': repr,
 
     'upper': lambda x: x.upper(),
     'lower': lambda x: x.lower(),
@@ -1464,7 +1496,7 @@ class Template(object):
       self.execute(data_dict, tokens.append, group=group,
                    trace=trace)
 
-    return ''.join(tokens)
+    return JoinTokens(tokens)
 
   def tokenstream(self, data_dict):
     """Yields a list of tokens resulting from expansion.
@@ -1547,6 +1579,26 @@ def MakeTemplateGroup(group):
     #t._CheckRefs()
 
 
+def JoinTokens(tokens):
+  """Join tokens (which may be a mix of unicode and str values).
+
+  See notes on unicode at the top.  This function allows mixing encoded utf-8
+  byte string tokens with unicode tokens.  (Python's default encoding is ASCII,
+  and we don't want to change that.)
+
+  We also want to support pure byte strings, so we can't get rid of the
+  try/except.  Two tries necessary.
+
+  If someone really wanted to use another encoding, they could monkey patch
+  jsontemplate.JoinTokens (this function).
+  """
+  try:
+    return ''.join(tokens)
+  except UnicodeDecodeError:
+    # This can still raise UnicodeDecodeError if that data isn't utf-8.
+    return ''.join(t.decode('utf-8') for t in tokens)
+
+
 def _DoRepeatedSection(args, context, callback, trace):
   """{.repeated section foo}"""
 
@@ -1652,7 +1704,7 @@ def _DoSubstitute(args, context, callback, trace):
           # We have more formatters to apply, so explicitly construct 'value'
           tokens = []
           template.execute(value, tokens.append, trace=trace)
-          value = ''.join(tokens)
+          value = JoinTokens(tokens)
 
       elif formatter_type == ENHANCED_FUNC:
         value = f(value, context, args)
@@ -1771,4 +1823,4 @@ def expand_with_style(template, style, data, body_subtree='body'):
     tokens = []
     execute_with_style_LEGACY(template, style, data, tokens.append,
                               body_subtree=body_subtree)
-    return ''.join(tokens)
+    return JoinTokens(tokens)
